{% extends "base.html" %}

{% block extrahead %}
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/firacode/6.2.0/fira_code.min.css" integrity="sha512-MbysAYimH1hH2xYzkkMHB6MqxBqfP0megxsCLknbYqHVwXTCg9IqHbk+ZP/vnhO8UEW6PaXAkKe2vQ+SWACxxA==" crossorigin="anonymous" referrerpolicy="no-referrer" />
<!-- Fathom - beautiful, simple website analytics -->
<script src="https://cdn.usefathom.com/script.js" data-site="TAUKXRLQ" defer></script>
<!-- / Fathom -->


<meta name="twitter:card" content="summary">
<meta name="twitter:site" content="@textualizeio">
<meta name="twitter:creator" content="@willmcgugan">
<meta property="og:title" content="Textual - {{ page.title }}">
<meta property="og:type" content="article">
<meta property="og:url" content="{{ page.canonical_url | url }}">
<meta property="og:site_name" content="Textual Documentation">
<meta property="og:description" content="Textual is a TUI framework for Python, inspired by modern web development.">
<meta property="og:image" content="https://raw.githubusercontent.com/Textualize/textual/main/imgs/textual.png">

<style>

    @font-face {
        font-family: "Virgil";
        src: url("https://unpkg.com/@excalidraw/excalidraw@0.12.0/dist/excalidraw-assets/Virgil.woff2");
      }
      @font-face {
        font-family: "Cascadia";
        src: url("https://unpkg.com/@excalidraw/excalidraw@0.12.0/dist/excalidraw-assets/Cascadia.woff2");
      }

      

</style>

<script lang="js">
window.onload = async (event) => {
  const demos = document.querySelectorAll(".textual-web-demo");

  if (demos.length){
    await fetch("https://textual-web-demos.fly.dev/health-check/", {
      method: 'GET',
      mode: 'no-cors'
    });
    demos.forEach((element)=>{
      const app = element.dataset.app;
      const iframe_html = `<div class="textual-web-demo"><iframe width="100%" onload="this.parentElement.classList.add('-loaded');" src="https://textual-web.io/textualize/${app}?delay=y&fontsize=12&ping=https://textual-web-demos.fly.dev/health-check/"></iframe></div>`;
      element.outerHTML = iframe_html;
    });
  }
}

</script>

{% endblock %}
